@@ -1,7 +1,9 @@
 # Configuration settings for a single directory Environment.
 # @api private
 class Puppet::Settings::EnvironmentConf
-  VALID_SETTINGS = [:modulepath, :manifest, :config_version, :environment_timeout, :parser].freeze
+
+  ENVIRONMENT_CONF_ONLY_SETTINGS = [:modulepath, :manifest, :config_version].freeze
+  VALID_SETTINGS = (ENVIRONMENT_CONF_ONLY_SETTINGS + [:environment_timeout, :environment_data_provider, :static_catalogs, :rich_data]).freeze
 
   # Given a path to a directory environment, attempts to load and parse an
   # environment.conf in ini format, and return an EnvironmentConf instance.
@@ -20,7 +22,6 @@ class Puppet::Settings::EnvironmentConf
   def self.load_from(path_to_env, global_module_path)
     path_to_env = File.expand_path(path_to_env)
     conf_file = File.join(path_to_env, 'environment.conf')
-    config = nil
 
     begin
       config = Puppet.settings.parse_file(conf_file)
@@ -37,8 +38,8 @@ class Puppet::Settings::EnvironmentConf
   # Configuration values are exactly those returned by the environment object,
   # without interpolation.  This is a special case for the default configured
   # environment returned by the Puppet::Environments::StaticPrivate loader.
-  def self.static_for(environment, parser, environment_timeout = 0)
-    Static.new(environment, environment_timeout, parser)
+  def self.static_for(environment, environment_timeout = 0, static_catalogs = false, rich_data = false)
+    Static.new(environment, environment_timeout, static_catalogs, nil, rich_data)
   end
 
   attr_reader :section, :path_to_env, :global_modulepath
@@ -64,13 +65,11 @@ class Puppet::Settings::EnvironmentConf
     if disable_per_environment_manifest
       environment_conf_manifest = absolute(raw_setting(:manifest))
       if environment_conf_manifest && fallback_manifest_directory != environment_conf_manifest
-        errmsg = ["The 'disable_per_environment_manifest' setting is true, but the",
-        "environment located at #{@path_to_env} has a manifest setting in its",
-        "environment.conf of '#{environment_conf_manifest}' which does not match",
-        "the default_manifest setting '#{puppet_conf_manifest}'. If this",
-        "environment is expecting to find modules in",
-        "'#{environment_conf_manifest}', they will not be available!"]
-        Puppet.err(errmsg.join(' '))
+        #TRANSLATORS 'disable_per_environment_manifest' is a setting and 'environment.conf' is a file name and should not be translated
+        message = _("The 'disable_per_environment_manifest' setting is true, but the environment located at %{path_to_env} has a manifest setting in its environment.conf of '%{environment_conf}' which does not match the default_manifest setting '%{puppet_conf}'.") %
+            { path_to_env: @path_to_env, environment_conf: environment_conf_manifest, puppet_conf: puppet_conf_manifest }
+        message += ' ' + _("If this environment is expecting to find modules in '%{environment_conf}', they will not be available!") % { environment_conf: environment_conf_manifest }
+        Puppet.err(message)
       end
       fallback_manifest_directory.to_s
     else
@@ -90,18 +89,30 @@ class Puppet::Settings::EnvironmentConf
     end
   end
 
+  def environment_data_provider
+    get_setting(:environment_data_provider, Puppet.settings.value(:environment_data_provider)) do |value|
+      value
+    end
+  end
+
   def modulepath
     default_modulepath = [File.join(@path_to_env, "modules")] + @global_module_path
     get_setting(:modulepath, default_modulepath) do |modulepath|
       path = modulepath.kind_of?(String) ?
         modulepath.split(File::PATH_SEPARATOR) :
         modulepath
-      path.map { |p| absolute(p) }.join(File::PATH_SEPARATOR)
+      path.map { |p| expand_glob(absolute(p)) }.flatten.join(File::PATH_SEPARATOR)
     end
   end
 
-  def parser
-    get_setting(:parser, Puppet.settings.value(:parser)) do |value|
+  def rich_data
+    get_setting(:rich_data, Puppet.settings.value(:rich_data)) do |value|
+      value
+    end
+  end
+
+  def static_catalogs
+    get_setting(:static_catalogs, Puppet.settings.value(:static_catalogs)) do |value|
       value
     end
   end
@@ -124,13 +135,25 @@ class Puppet::Settings::EnvironmentConf
     section_keys = config.sections.keys
     main = config.sections[:main]
     if section_keys.size > 1
-      Puppet.warning("Invalid sections in environment.conf at '#{path_to_conf_file}'. Environment conf may not have sections. The following sections are being ignored: '#{(section_keys - [:main]).join(',')}'")
+      # warn once per config file path
+      Puppet.warn_once(
+        :invalid_settings_section, "EnvironmentConf-section:#{path_to_conf_file}",
+        _("Invalid sections in environment.conf at '%{path_to_conf_file}'. Environment conf may not have sections. The following sections are being ignored: '%{sections}'") % {
+          path_to_conf_file: path_to_conf_file,
+          sections: (section_keys - [:main]).join(',')
+        })
       valid = false
     end
 
     extraneous_settings = main.settings.map(&:name) - VALID_SETTINGS
     if !extraneous_settings.empty?
-      Puppet.warning("Invalid settings in environment.conf at '#{path_to_conf_file}'. The following unknown setting(s) are being ignored: #{extraneous_settings.join(', ')}")
+      # warn once per config file path
+      Puppet.warn_once(
+        :invalid_settings, "EnvironmentConf-settings:#{path_to_conf_file}",
+        _("Invalid settings in environment.conf at '%{path_to_conf_file}'. The following unknown setting(s) are being ignored: %{ignored_settings}") % {
+          path_to_conf_file: path_to_conf_file,
+          ignored_settings: extraneous_settings.join(', ')
+        })
       valid = false
     end
 
@@ -139,17 +162,26 @@ class Puppet::Settings::EnvironmentConf
 
   def get_setting(setting_name, default = nil)
     value = raw_setting(setting_name)
-    value ||= default
+    value = default if value.nil?
     yield value
   end
 
+  def expand_glob(path)
+    return nil if path.nil?
+    if path =~ /[*?\[\{]/
+      Dir.glob(path)
+    else
+      path
+    end
+  end
+
   def absolute(path)
     return nil if path.nil?
     if path =~ /^\$/
       # Path begins with $something interpolatable
       path
     else
-      File.expand_path(path, @path_to_env)
+      Puppet::FileSystem.expand_path(path, @path_to_env)
     end
   end
 
@@ -158,12 +190,20 @@ class Puppet::Settings::EnvironmentConf
   # @api private
   class Static
     attr_reader :environment_timeout
-    attr_reader :parser
+    attr_reader :environment_data_provider
+    attr_reader :rich_data
+    attr_reader :static_catalogs
 
-    def initialize(environment, environment_timeout, parser)
+    def initialize(environment, environment_timeout, static_catalogs, environment_data_provider = nil, rich_data = false)
       @environment = environment
       @environment_timeout = environment_timeout
-      @parser = parser
+      @static_catalogs = static_catalogs
+      @environment_data_provider = environment_data_provider
+      @rich_data = rich_data
+    end
+
+    def path_to_env
+      nil
     end
 
     def manifest
